SystemOrganization addCategory: #'P4-SystemInstaller'!

P4Object subclass: #P4ClassInstaller
	instanceVariableNames: 'methodInstaller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

!P4ClassInstaller commentStamp: '<historical>' prior: 0!
The ClassInstaller is responisble for compiling sources and create proper binary/executable formats. Most of its actions are delegated to helper classes.!

P4ClassInstaller subclass: #P4DynamicClassInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

!P4DynamicClassInstaller commentStamp: '<historical>' prior: 0!
I am a class installer used by the running system. 

The DynamicClassInstaller can compile and install new classes into a running system.!

P4ClassInstaller subclass: #P4StaticClassInstaller
	instanceVariableNames: 'dumper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

!P4StaticClassInstaller commentStamp: '<historical>' prior: 0!
I compile classes to static libraries which can be linked together.

Example:
P4StaticClassInstaller exportKernelTestOSX64.

This will compile the pinocchio kernele and output .o files in the current image directory. Those binaries should be linked together with the core sources (which contain some basic primitives)!

!P4StaticClassInstaller class methodsFor: 'exporting' stamp: 'CamilloBruni 2/3/2012 16:58'!
exportKernelTestLinux64
	^ self linux64
		compileClasses: self kernelTestExportClasses! !

!P4StaticClassInstaller class methodsFor: 'exporting' stamp: 'CamilloBruni 2/3/2012 16:58'!
exportKernelTestOSX64
	^ self osx64
		compileClasses: self kernelTestExportClasses! !

!P4StaticClassInstaller class methodsFor: 'exporting' stamp: 'CamilloBruni 2/3/2012 16:58'!
kernelTestExportClasses
	^ {P4StaticKernelTest. P4KernelTest}! !

!P4StaticClassInstaller class methodsFor: 'instance creation' stamp: 'CamilloBruni 3/27/2012 17:14'!
linux64
	^ self os: P4Linux x86_64! !

!P4StaticClassInstaller class methodsFor: 'instance creation' stamp: 'CamilloBruni 3/27/2012 17:15'!
os: aP4OS
	^ self new
		os: aP4OS;
		yourself! !

!P4StaticClassInstaller class methodsFor: 'instance creation' stamp: 'CamilloBruni 3/27/2012 17:14'!
osx64
	^ self os: P4OSX x86_64! !

!P4StaticClassInstaller methodsFor: 'accessing' stamp: 'CamilloBruni 2/6/2012 16:58'!
baseClasses
	^ 
	{
		P4SymbolTable.
		P4Symbol.
		P4RootPackage.
		P4Package.
		P4ClassReference.
		P4MethodDictionary.
		P4Method.
		P4Metaclass.
		P4True.
		P4False.
		P4UndefinedObject.
		P4SmallInteger.
		P4Closure.
		P4SetBucket.
		P4SmalltalkBehavior.
		P4String.
		P4Array.
		P4RemoteArray.
		P4DoesNotUnderstand.
		P4Object.
	}.! !

!P4StaticClassInstaller methodsFor: 'compiling' stamp: 'CamilloBruni 4/10/2012 18:39'!
compileClass: compiledClass
	self flag: 'clean up pathOfClass stuff'.
	
	self compileInstanceMethodsOf: compiledClass.
	self compileClassMethodsOf: compiledClass.
	
	dumper
		dump: compiledClass;
		dump: compiledClass theBehavior;
		dump: compiledClass pclass;
		dump: compiledClass pclass theBehavior;
		into: (self pathOfClass: compiledClass inPharo)! !

!P4StaticClassInstaller methodsFor: 'compiling' stamp: 'CamilloBruni 4/10/2012 18:38'!
compileClassMethodsOf: compiledClass
	compiledClass pclass inPharo methodDictionary
		do: [ :aMethod | methodInstaller installMethod: aMethod on: compiledClass pclass ]! !

!P4StaticClassInstaller methodsFor: 'compiling' stamp: 'CamilloBruni 4/10/2012 18:36'!
compileClasses: classes
	P4RootPackage flushPackages.
	dumper dumpCoreObjects.

	dumper addPharoClasses: classes.
	dumper addPharoClasses: self baseClasses.

	dumper do: [ :toDump|
		Transcript show: toDump; show: ' '; show: dumper todo size; cr.
		self compileClass: toDump ].
	
	dumper dumpPackages.
	dumper dumpSymbolTable.! !

!P4StaticClassInstaller methodsFor: 'compiling' stamp: 'CamilloBruni 4/10/2012 18:39'!
compileInstanceMethodsOf: compiledClass
	compiledClass inPharo methodDictionary do: [ :aMethod | 
		methodInstaller installMethod: aMethod on: compiledClass ]! !

!P4StaticClassInstaller methodsFor: 'accessing' stamp: 'OliFlueckiger 7/7/2011 17:45'!
initialize 
	super initialize.
	methodInstaller := P4StaticMethodInstaller new.! !

!P4StaticClassInstaller methodsFor: 'accessing' stamp: 'OliFlueckiger 7/5/2011 15:56'!
loadClass: aClass
	<preventExport>
	^ P4Package loadClass: aClass! !

!P4StaticClassInstaller methodsFor: 'accessing' stamp: 'CamilloBruni 3/29/2012 13:27'!
os: anOs
	dumper := P4BinaryOut new os: anOs.
	methodInstaller asm: anOs asm.! !

!P4StaticClassInstaller methodsFor: 'accessing' stamp: 'OliFlueckiger 6/23/2011 17:16'!
pathOfClass: aClass
	<preventExport>
	| package |
	package := P4Package packageOfClass: aClass.
	package ensureDirectory.
	^ package relativePathNameFor: aClass pname! !

P4Object subclass: #P4MethodInstaller
	instanceVariableNames: 'compiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

P4MethodInstaller subclass: #P4DynamicMethodInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

!P4MethodInstaller methodsFor: 'accessing' stamp: 'CamilloBruni 3/29/2012 13:40'!
asm: anAssembler
	compiler := self newCompiler.
	"anAssembler installer: self."
	compiler asm: anAssembler.! !

!P4MethodInstaller methodsFor: 'accessing' stamp: 'CamilloBruni 3/29/2012 13:27'!
processor: aProcessor
	compiler := self newCompiler.
	aProcessor installer: self.
	compiler asm: aProcessor.! !

P4MethodInstaller subclass: #P4StaticMethodInstaller
	instanceVariableNames: 'globals references'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'P4-SystemInstaller'!

!P4StaticMethodInstaller methodsFor: 'as yet unclassified' stamp: 'OliFlueckiger 7/8/2011 10:49'!
flush
	globals := P4IdentityDictionary new.
	references := P4IdentityDictionary new.! !

!P4StaticMethodInstaller methodsFor: 'as yet unclassified' stamp: 'OliFlueckiger 7/15/2011 17:07'!
global: aGlobal at: position
	globals at: position put: aGlobal! !

!P4StaticMethodInstaller methodsFor: 'as yet unclassified' stamp: 'OliFlueckiger 7/14/2011 19:03'!
installFunction: functionName at: position
	functionName isSymbol ifFalse: [ self error: 'not a function call' ].
	references at: position put: functionName! !

!P4StaticMethodInstaller methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 2/6/2012 17:35'!
installMethod: aCompiledMethod on: aClass
	self flush.
	compiler host: aClass.
	(compiler compileMethod: aCompiledMethod ) ifNotNilDo: [ :method |
		method usedGlobals: globals.
		method usedReferences: references.
		aClass installMethod: method.]! !

!P4StaticMethodInstaller methodsFor: 'as yet unclassified' stamp: 'ToonVerwaest 6/25/2011 14:21'!
newCompiler
	^ P4SmalltalkKernelCompiler new! !
